import six

from django import forms
from elasticutils import S

from .fields import BaseField


class FilterFormOptions(object):
    def __init__(self, name, options):

        self.s = options.pop('s', S())

        if options != {}:
            raise NameError("Unknown option {0} in {1}.FilterFormMeta".format(
                ', '.join(options.keys()), name))


class FilterFormMetaclass(type(forms.Form)):
    def __new__(mcs, name, bases, attrs):
        if 'FilterFormMeta' in attrs:
            options_attr = attrs.pop('FilterFormMeta')
            options = dict((key, value)
                           for key, value in options_attr.__dict__.items()
                           if key[0] != '_')
            attrs['_filterformmeta'] = FilterFormOptions(name, options)

        return super(FilterFormMetaclass, mcs).__new__(mcs, name, bases, attrs)


class FilterFormBase(forms.Form):

    def __init__(self, data=None, files=None, s=None, **kwargs):
        super(FilterFormBase, self).__init__(data=data, files=files, **kwargs)

        self.s = s or self._filterformmeta.s

    def search(self):
        if not self.is_valid():
            raise ValueError('Can not search with invalid data')

        s = self.s
        cleaned_data = self.cleaned_data

        for name, field in self.get_filter_fields():
            s = field.search(s, name, cleaned_data)

        return s

    def get_filter_fields(self):
        for (name, field) in self.fields.items():
            if isinstance(field, BaseField):
                yield (name, field)


class FilterForm(six.with_metaclass(FilterFormMetaclass, FilterFormBase)):
    pass



from django.forms import fields


class BaseField(fields.Field):

    def search(self, s, name, data):
        raise NotImplementedError


class Query(BaseField):

    fields = ['_all']

    def __init__(self, fields=None, **kwargs):
        super(Query, self).__init__(**kwargs)
        if fields is not None:
            self.fields = fields

    def search(self, s, name, data):
        value = data[name]
        if not value:
            return s

        fields = self.fields
        if len(fields) == 1:
            query = {
                'match': {
                    fields[0]: value,
                }
            }
        else:
            query = {
                'multi_match': {
                    'query': value,
                    'fields': fields,
                }
            }

        return s.query_raw(query)


class Filter(BaseField):

    field = None

    def __init__(self, field=None, **kwargs):
        super(Filter, self).__init__(**kwargs)
        if field is not None:
            self.field = field

    def search(self, s, name, data):
        field = self.field or name
        value = data[name]
        s.filter(**{field: value})
